package ru.dbotthepony.mc.otm.storage;

import net.minecraft.MethodsReturnNonnullByDefault;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import java.util.HashMap;
import java.util.Objects;

@ParametersAreNonnullByDefault
@MethodsReturnNonnullByDefault
public class StorageObjectRegistry {
	private static final HashMap<Class<? extends IStorageStack>, StorageObjectTuple<? extends IStorageStack>> REGISTRY = new HashMap<>();

	public static <T extends IStorageStack> boolean register(Class<T> identity, T empty) {
		final var tuple = new StorageObjectTuple<>(identity, empty);
		final boolean removed = REGISTRY.remove(identity) != null;
		REGISTRY.put(identity, tuple);

		return removed;
	}

	@SuppressWarnings("unchecked")
	@Nullable
	public static <T extends IStorageStack> StorageObjectTuple<T> get(Class<T> identity) {
		return (StorageObjectTuple<T>) REGISTRY.get(identity);
	}

	@Nonnull
	public static <T extends IStorageStack> StorageObjectTuple<T> getOrError(Class<T> identity) {
		return Objects.requireNonNull(get(identity), "No storage mapping present for " + identity.toString());
	}
}
